import { FluxStandardAction } from "flux-standard-action";
import { Reducer, ReducersMapObject } from "redux";
import { Effect } from "redux-saga/effects";
declare type ReadonlyRecord<K extends string, T> = {
    readonly [P in K]: T;
};
declare type SideEffectRecord<A = any> = ReadonlyRecord<string, (...args: readonly A[]) => unknown>;
declare type ActionTypes<A extends SideEffectRecord<unknown>> = {
    readonly [P in keyof A]: readonly [string, string, string, string];
};
export declare type SagaIterator<RT> = Generator<Effect<unknown>, RT, unknown>;
declare type Action = FluxStandardAction<string, {
    readonly result?: unknown;
    readonly error?: unknown;
}>;
declare type SideEffectAction<A extends SideEffectRecord<unknown>, B extends ActionTypes<A>, P extends keyof A> = {
    readonly type: B[P][0];
    readonly payload: {
        readonly params: Parameters<A[P]>;
    };
};
declare type SagaActions<A extends SideEffectRecord<unknown>, B extends ActionTypes<A>, ErrorType extends unknown = Error> = {
    readonly [P in keyof A]: readonly [(...params: Parameters<A[P]>) => SideEffectAction<A, B, P>, (result: ReturnType<A[P]> extends Promise<infer Result> ? Result : ReturnType<A[P]>) => {
        readonly type: B[P][1];
        readonly payload: {
            readonly result: ReturnType<A[P]> extends Promise<infer Result> ? Result : ReturnType<A[P]>;
        };
    }, (error: ErrorType) => {
        readonly type: B[P][2];
        readonly payload: {
            readonly error: ErrorType;
        };
        readonly error: true;
    }, Reducer<object, Action>, (action: SideEffectAction<A, B, P>) => SagaIterator<void>];
};
declare type SagaBoilerplate<A extends SideEffectRecord<unknown>, B extends ActionTypes<A>, ErrorType extends unknown = Error> = {
    readonly actions: SagaActions<A, B, ErrorType>;
    readonly rootReducer: ReducersMapObject;
    readonly rootSaga: () => SagaIterator<void>;
};
export declare const concoctBoilerplate: <A extends ReadonlyRecord<string, (...args: readonly any[]) => unknown>, B extends ActionTypes<A>, C = {}>(sideEffects: A, actionTypes: B, defaultState?: C | undefined) => SagaBoilerplate<A, B, Error>;
export {};
